@@ -14,7 +14,7 @@
 
 	The author may be reached as becker@scyld.com, or C/O
 	 Scyld Computing Corporation
-	 410 Severn Ave., Suite 210
+	 914 Bay Ridge Road, Suite 220
 	 Annapolis MD 21403
 
 	Support and updates available at
@@ -27,43 +27,44 @@
 */
 
 static char *version_msg =
-"eepro100-diag.c:v2.12 4/15/2003 Donald Becker (becker@scyld.com)\n"
+"eepro100-diag.c:v2.13 2/28/2005 Donald Becker (becker@scyld.com)\n"
 " http://www.scyld.com/diag/index.html\n";
 static char *usage_msg =
 "Usage: eepro100-diag [-aDEefFGhmqrRtvVwW] [-p <IOport>] [-[AF] <media>]\n"
 "  For details and other options see http://www.scyld.com/diag/index.html\n";
 
 static const char long_usage_msg[] =
-"Usage: %s [-aDfrRvVw] [-AF <speed+duplex>] [-#<BoardNum>]
-
-   Show the internal state of a network adapter.
-
-   The common usage is
-      diag -aem
-
- Frequently used options are
-   -a  --show_all_registers	Print all registers.
-   -e  --show-eeprom	Dump EEPROM contents, \"-ee\" shows the details.
-   -m  --show_mii		Print the MII transceiver state
-					Using -mm monitors the link.
-   -f  --force		Perform operation, even on a running NIC.
-
- To operate on a single NIC, or one that hasn't been automatically found:
-   -#  --card_num	INDEX	Operate on the specified card index.
-   -p  --port-base	IOADDR	Assume an adapter at the specified I/O address.
-   -t  --chip-type	TYPE	Specify adapter type (with '-p'), use '-1' to list.
-
- To change the persistent EEPROM settings
-   -G  --parameters PARMS	Set adapter-specific parameters.
-   -H  --new-hwaddr 01:23:45:67:89:ab
-        Set a new hardware station address.  Typically disabled for safety.
-   -w  --write-EEPROM	 Actually write the new settings into the EEPROM.
-
-   -D  --debug
-   -v  --verbose	Report each action taken.
-   -V  --version	Emit version information.
-
-   -A  --advertise <speed|setting>  (See the mii-diag manual page.)
+"Usage: %s [-aDfrRvVw] [-AF <speed+duplex>] [-#<BoardNum>]\n\
+\n\
+   Show the internal state of a network adapter.\n\
+\n\
+   The common usage is\n\
+      eepro100-diag -aem\n\
+\n\
+ Frequently used options are\n\
+   -a  --show_all_registers	Print all registers.\n\
+   -e  --show-eeprom	Dump EEPROM contents, \"-ee\" shows the details.\n\
+   -m  --show_mii		Print the MII transceiver state\n\
+					Using -mm monitors the link.\n\
+   -f  --force		Perform operation, even on a running NIC.\n\
+\n\
+ To operate on a single NIC, or one that hasn't been automatically found:\n\
+   -#  --card_num	INDEX	Operate on the specified card index.\n\
+   -p  --port-base	IOADDR	Assume an adapter at the specified I/O address.\n\
+   -t  --chip-type	TYPE	Specify adapter type (with '-p').  Use '-1' to\n\
+                            list available types indicies.\n\
+\n\
+ To change the persistent EEPROM settings\n\
+   -G  --parameters PARMS	Set adapter-specific parameters.\n\
+   -H  --new-hwaddr 01:23:45:67:89:ab\n\
+        Set a new hardware station address.  Typically disabled for safety.\n\
+   -w  --write-EEPROM	 Actually write the new settings into the EEPROM.\n\
+\n\
+   -D  --debug\n\
+   -v  --verbose	Report each action taken.\n\
+   -V  --version	Emit version information.\n\
+\n\
+   -A  --advertise <speed|setting>  (See the mii-diag manual page.)\n\
 ";
 
 #if ! defined(__OPTIMIZE__)
@@ -90,6 +91,9 @@ static const char long_usage_msg[] =
 /* No libmii.h or libflash.h yet, thus the declarations here. */
 extern int show_mii_details(long ioaddr, int phy_id);
 extern int monitor_mii(long ioaddr, int phy_id);
+/* Simple alternate functions if libmii is not used. */
+extern int monitor_mii(long ioaddr, int phy_id) __attribute__((weak));
+extern int show_mii_details(long ioaddr, int phy_id) __attribute__((weak));
 
 extern int flash_pci_rom_show(long addr_ioaddr, long data_ioaddr);
 extern int flash_pci_rom_dump(long addr_ioaddr, long data_ioaddr,
