@@ -9,7 +9,7 @@
 
 	The author may be reached as becker@scyld.com, or C/O
 	 Scyld Computing Corporation
-	 410 Severn Ave., Suite 210
+	 914 Bay Ridge Road, Suite 220
 	 Annapolis MD 21403
 
 	Support and updates available at
@@ -20,7 +20,7 @@
 	http://www.ssti.com/products/products.html
 */
 static const char version_msg[] =
-"libflash.c:v2.06 4/22/2003 Copyright Donald Becker, becker@scyld.com";
+"libflash.c:v2.06a 7/4/2003 Copyright Donald Becker, becker@scyld.com";
 
 /* External interface
       flash_show(), flash_dump() and flash_program()
@@ -92,6 +92,7 @@ struct flash_id_table {
 	{ 0x1F, 0xC4, 0, 512*1024, 256, "Atmel AT29LV040A", },
 	/* Bulk-erase, byte-program parts. */
 	{ 0x1F, 0x03, ERASE_FIRST, 64*1024,   1, "Atmel AT49BV512", },
+	{ 0xBF, 0xD4, ERASE_FIRST, 64*1024,   1, "Silicon Storage Technology SST39VF512", },
 	/* AMD parts that require 90/01 read method. */
 	{ 0x01, 0x6D, 0, 128*1024, 8*1024, "AMD Am29LV001BB", },
 	{ 0x01, 0xED, 0, 128*1024, 8*1024, "AMD Am29LV001BT", },
@@ -133,6 +134,7 @@ struct flash_id_table {
 	{ 0x01, 0xA3, 0, 0, 0, "AMD Am29LV033C", },
 	{ 0x01, 0xC8, 0, 0, 0, "AMD Am29LV017D", },
 	{ 0x01, 0xD7, 0, 0, 0, "AMD Am29LV640D", },
+	{ 0xDA, 0xC1, 0, 128*1024, 128, "Winbond W29EE011", },
 	{ 0,},
 };
 
@@ -333,6 +335,9 @@ void sst_flash_clear(long addr_ioaddr, long data_ioaddr)
 	do_cmd(addr_ioaddr, data_ioaddr, 0x80);
 	do_cmd(addr_ioaddr, data_ioaddr, 0x10);
 	usleep(20000);				/* 20 msec. */
+	/* While SST39VF512 specifies a max tsce of 100 ms, the */
+	/* AT49BV512 specifies a tEC Erase Cycle Time of 10 sec, so ... */
+	sleep(10);
 }
 
 int flash_program(long addr_ioaddr, long data_ioaddr, char *filename)
