@@ -3,7 +3,7 @@
    This is a diagnostic and EEPROM setup program for PCI NE2000 Ethernet
    adapters.
 
-	Copyright 1998-2001 by Donald Becker.
+	Copyright 1998-2002 by Donald Becker.
 	This software may be used and distributed according to the terms of
 	the GNU General Public License (GPL), incorporated herein by reference.
 	Contact the author for use under other terms.
@@ -13,7 +13,7 @@
 
 	The author may be reached as becker@scyld.com, or C/O
 	 Scyld Computing Corporation
-	 410 Severn Ave., Suite 210
+	 914 Bay Ridge Road, Suite 220
 	 Annapolis MD 21403
 
 	Support and updates available at
@@ -23,7 +23,7 @@
 	http://www.scyld.com/diag/index.html
 	RealTek RTL8029AS datasheet, available from www.RealTek.com.tw.
 	Holtek HT80232 ECON-PCI datasheet, www.holtek.com.tw
-	http://www.winbond.com.tw/produ/w89c940f.htm
+	http://www.winbond.com.tw/PDF/sheet/w89c940f.pdf
 	http://www.VIA.com.tw/products/prodamazon.htm
 
 	Common-sense licensing statement: Using any portion of this program in
@@ -32,7 +32,7 @@
 */
 
 static char *version_msg =
-"ne2k-pci-diag.c:v2.01 1/8/2001 Donald Becker (becker@scyld.com)\n"
+"ne2k-pci-diag.c:v2.06 2/28/2005 Donald Becker (becker@scyld.com)\n"
 " http://www.scyld.com/diag/index.html\n";
 static char *usage_msg =
 "Usage: ne2k-pci-diag [-aEefFmqrRtvVwW] [-p <IOport> [-t <chip-type>]]\n";
@@ -75,7 +75,10 @@ struct option longopts[] = {
 	{ 0, 0, 0, 0 }
 };
 
-enum ne2k_chip_flags { ONLY_16BIT_IO=1, HOLTEK_EE=2, RTL_EE=4, WINBOND_EE=8,};
+enum ne2k_chip_flags {
+	ONLY_16BIT_IO=1, HOLTEK_EE=2, RTL_EE=4, WINBOND_EE=8, VIA_EE=16,
+	BOGUS_WINBOND=0x100,
+};
 extern int    ne2k_diag(int vend_id, int dev_id, long ioaddr, int part_idx);
 
 /* The table of known chips.
@@ -93,26 +96,29 @@ struct pcidev_entry {
 } pcidev_tbl[] = {
 	{ "Generic PCI NE2000 clone", "Generic PCI NE2000 clone",
 	  0xffff, 0x2000, 0xffff, 0, 32, ne2k_diag},
+	/* First three are ordered for the 'unknown_eeprom_message' message. */
 	{ "Realtek 8029", "RealTek RTL8029",
 	  0x10ec, 0x8029, 0xffff, RTL_EE, 32, ne2k_diag},
 	{ "Winbond 89C940", "Winbond 89C940",
 	  0x1050, 0x0940, 0xffff, WINBOND_EE, 32, ne2k_diag},
+	{ "Holtek HT80229", "Holtek HT80229",
+	  0x12c3, 0x5598, 0xffff, HOLTEK_EE, 64, ne2k_diag},
+	{ "Via 82C926", "Via 82C926",
+	  0x1106, 0x0926, 0xffff, VIA_EE, 32, ne2k_diag},
 	{ "Compex RL2000", "Compex RL2000",
 	  0x11f6, 0x1401,  0xffff, 0, 32, ne2k_diag},
 	{ "KTI ET32P2", "KTI ET32P2",
 	  0x8e2e, 0x3000, 0xffff, 0, 32, ne2k_diag},
 	{ "NetVin NV5000", "NetVin NV5000",				/* Renumbered RTL8029. */
 	  0x4a14, 0x5000, 0xffff, RTL_EE, 32, ne2k_diag},
-	{ "Via 82C926", "Via 82C926",
-	  0x1106, 0x0926, 0xffff, 0, 32, ne2k_diag},
 	{ "SureCom NE34", "SureCom NE34",
 	  0x10bd, 0x0e34, 0xffff, 0, 32, ne2k_diag},
 	{ "Winbond 940 (unprogrammed)","Winbond 940 (unprogrammed)",
 	  0x1050, 0x5a5a,  0xffff, WINBOND_EE, 32, ne2k_diag},
-	{ "Holtek HT80229", "Holtek HT80229",
-	  0x12c3, 0x5598, 0xffff, HOLTEK_EE, 64, ne2k_diag},
 	{ "Holtek HT80232", "Holtek HT80232",
 	  0x12c3, 0x0058, 0xffff, ONLY_16BIT_IO | HOLTEK_EE, 64, ne2k_diag},
+	{ "Winbond 89C940 (misprogrammed)", 0,
+	  0x8c4a, 0x1980, 0xffff, WINBOND_EE | BOGUS_WINBOND, 32, ne2k_diag},
 	{ 0, }
 };
 
@@ -127,6 +133,7 @@ unsigned int opt_GPIO = 0;		/* General purpose I/O setting. */
 int do_write_eeprom = 0, do_test = 0;
 int new_default_media = -1;
 static unsigned char new_hwaddr[6], set_hwaddr = 0;
+static int emergency_rewrite = 0;
 
 static int scan_proc_pci(int card_num);
 static int get_media_index(const char *name);
@@ -137,7 +144,6 @@ main(int argc, char **argv)
 {
 	int port_base = 0, chip_type = 0;
 	int errflag = 0, show_version = 0;
-	int emergency_rewrite = 0;
 	int c, longind;
 	int card_num = 0;
 	extern char *optarg;
@@ -156,7 +162,7 @@ main(int argc, char **argv)
 			if (new_default_media < 0)
 				errflag++;
 			break;
-		case 'G': opt_G++; opt_GPIO = strtol(optarg, NULL, 16); break;
+		case 'G': opt_G++; opt_GPIO = strtoul(optarg, NULL, 16); break;
 		case 'H': {
 			int hwaddr[6], i;
 			if (sscanf(optarg, "%2x:%2x:%2x:%2x:%2x:%2x",
@@ -171,7 +177,7 @@ main(int argc, char **argv)
 		}
 		case 'm': show_mii++;	 break;
 		case 'p':
-			port_base = strtol(optarg, NULL, 16);
+			port_base = strtoul(optarg, NULL, 16);
 			break;
 		case 'q': if (verbose) verbose--;		 break;
 		case 'r': opt_restart++;	break;
@@ -265,8 +271,9 @@ static int scan_proc_bus_pci(int card_num)
 				   &pci_bus, &pci_devid, &irq, &pciaddr0, &pciaddr1) <= 0)
 			break;
 		for (i = 0; pcidev_tbl[i].vendor; i++) {
-			if (pci_devid !=
-				(pcidev_tbl[i].vendor << 16) + pcidev_tbl[i].device)
+			if ((pci_devid >> 16) != pcidev_tbl[i].vendor
+				|| (pci_devid & pcidev_tbl[i].device_mask) !=
+				pcidev_tbl[i].device)
 				continue;
 			chip_idx = i;
 			card_cnt++;
@@ -488,19 +495,20 @@ static const char *const rtl_xcvr[4] = {
 static int rtl_romsize[4] = { 0, 8, 16, 32};
 
 
-static int do_eeprom_cmd(int chip_flags, long ioaddr, int cmd, int cmd_len);
+static void do_ne2k_eeprom(long ioaddr, int part_idx);
 static int read_eeprom(int chip_flags, long ioaddr, int offset, int addr_len);
 static int do_update(int chip_flags, long ioaddr, int index, int addr_len,
 					 const char *field_name, int old_value, int new_value);
 static void parse_8029_eeprom(unsigned short *ee_data);
 static void parse_holtek_eeprom(unsigned short *ee_data);
+static void parse_winbond_eeprom(unsigned short *ee_data);
 
 int ne2k_diag(int vend_id, int dev_id, long ioaddr, int part_idx)
 {
 	int chip_active = 0;
 	int saved_window = inb(E8390_CMD);
 	int capabilities = pcidev_tbl[part_idx].flags;
-	int i, ee_addr_len = 6;
+	int i;
 
 	/* It's mostly safe to examine the registers and EEPROM during
 	   operation.  But warn the user, and make then pass '-f'. */
@@ -551,65 +559,7 @@ int ne2k_diag(int vend_id, int dev_id, long ioaddr, int part_idx)
 		}
 	}
 
-	/* Read the EEPROM. */
-	if (capabilities & (RTL_EE | HOLTEK_EE)) {
-		if (capabilities & RTL_EE)
-			outb(E8390_PAGE3, ioaddr + E8390_CMD);
-		for (i = 0; i < EEPROM_SIZE; i++)
-			eeprom_contents[i] =
-				read_eeprom(capabilities, ioaddr, i, ee_addr_len);
-	}
-	if (capabilities & WINBOND_EE) {
-		outb(E8390_PAGE3, ioaddr + E8390_CMD);
-		for (i = 0; i < EEPROM_SIZE; i++) {
-			int boguscnt = 1000;
-			outb(0x80, ioaddr + 2);	/* Read command. */
-			outb(i, ioaddr + 4);	/* Address to read. */
-			for (boguscnt = 1000; boguscnt >= 0; boguscnt--)
-				if ((inb(ioaddr + 2) & 0x80) == 0)
-					break;
-			eeprom_contents[i] = inb(ioaddr + 4);
-		}
-	}
-
-	if (set_hwaddr &&  (capabilities & (RTL_EE | HOLTEK_EE))) {
-		int eeprom_sa_offset = capabilities & RTL_EE ? 1 : 0;
-		const char *const field_names[] = {
-			"MAC address 0/1", "MAC address 2/3", "MAC address 4/5", };
-		memcpy(new_ee_contents, eeprom_contents, sizeof(eeprom_contents));
-		for (i = 0; i < 3; i++) {
-			new_ee_contents[i + eeprom_sa_offset] =
-				new_hwaddr[i*2] + (new_hwaddr[i*2+1]<<8);
-		}
-		printf("Writing new MAC station address.\n");
-		for (i = eeprom_sa_offset; i < eeprom_sa_offset + 3; i++)
-			if (new_ee_contents[i] != eeprom_contents[i])
-				do_update(capabilities, ioaddr, i, ee_addr_len,
-						  field_names[i - eeprom_sa_offset],
-						  eeprom_contents[i], new_ee_contents[i]);
-		/* Re-read all contents. */
-		for (i = 0; i < EEPROM_SIZE; i++)
-			eeprom_contents[i] =
-				read_eeprom(capabilities, ioaddr, i, ee_addr_len);
-	}
-
-	if (show_eeprom) {
-		unsigned short sum = 0;
-		printf("EEPROM contents:");
-		for (i = 0; i < EEPROM_SIZE; i++) {
-			printf("%s %4.4x", (i & 7) == 0 ? "\n ":"",
-				   eeprom_contents[i]);
-			sum += eeprom_contents[i];
-		}
-		printf("\n The word-wide EEPROM checksum is %#4.4x.\n", sum);
-	}
-	/* The user will usually want to see the interpreted EEPROM contents. */
-	if (verbose || show_eeprom) {
-		if (capabilities & RTL_EE)
-			parse_8029_eeprom(eeprom_contents);
-		else if (capabilities & HOLTEK_EE)
-			parse_holtek_eeprom(eeprom_contents);
-	}
+	do_ne2k_eeprom(ioaddr, part_idx);
 
 	if (do_test) {
 		/* Run a loopback test. */
@@ -621,7 +571,9 @@ int ne2k_diag(int vend_id, int dev_id, long ioaddr, int part_idx)
 
 /* Serial EEPROM section. */
 
-/* First, the definitions for the HolTek ECON/PCI chip. */
+#include <ctype.h>
+
+/* First, the definitions for the serial EEPROM bits. */
 struct ee_ctrl_bits {
 	int offset;
 	unsigned char shift_clk,	/* Bit that drives SK (shift clock) pin */
@@ -631,6 +583,7 @@ struct ee_ctrl_bits {
 } ee_ctrl_tbl[] = {
 	{ 0x01,  0x04, 0x01, 0x88, 0x8A, 0x00 }, 			/* RTL8029 */
 	{ 0x22,  0x12, 0x04, 0x11, 0x19, 0x00 },			/* HolTek */
+	{ 0x10,  0x04, 0x01, 0x88, 0x8A, 0x00 },			/* VIA */
 };
 
 /* This executes a generic EEPROM command, typically a write or write enable.
@@ -679,12 +632,22 @@ static int eeprom_busy_poll(int chip_flags, long ioaddr)
 	return i;
 }
 
-/* The abstracted functions for EEPROM access. */
+/* The abstracted functions for serial EEPROM access. */
 static int read_eeprom(int chip_flags, long ioaddr, int location, int addr_len)
 {
-	return do_eeprom_cmd(chip_flags, ioaddr,
-						 ((EE_ReadCmd << addr_len) | location) << 16,
-						 3 + addr_len + 16) & 0xffff;
+	if (chip_flags & WINBOND_EE) {
+		int boguscnt;
+		outb(0x80, ioaddr + 2);			/* Read command. */
+		outw(location, ioaddr + 4);		/* Address to read. */
+		for (boguscnt = 10000; boguscnt >= 0; boguscnt--)
+			if ((inb(ioaddr + 2) & 0x80) == 0)
+				break;
+		inw(ioaddr + 4);
+		return inw(ioaddr + 4);
+	} else
+		return do_eeprom_cmd(chip_flags, ioaddr,
+							 ((EE_ReadCmd << addr_len) | location) << 16,
+							 3 + addr_len + 16) & 0xffff;
 }
 
 static void write_eeprom(int chip_flags, long ioaddr, int index,
@@ -692,6 +655,18 @@ static void write_eeprom(int chip_flags, long ioaddr, int index,
 {
 	int i;
 
+	if (chip_flags & WINBOND_EE) {	/* The chip handles the write for us. */
+		int boguscnt;
+		/* The chip should already be in PAGE3 from the read. */
+		outb(0xC0, ioaddr + 2);		/* Write command. */
+		outw(index, ioaddr + 4);	/* Address to write. */
+		outw(value, ioaddr + 4);	/* Data to write. */
+		for (boguscnt = 10000; boguscnt >= 0; boguscnt--)
+			if ((inb(ioaddr + 2) & 0x80) == 0)
+				break;
+		return;
+	}
+
 	/* Poll for previous op finished. */
 	eeprom_busy_poll(chip_flags, ioaddr);
 
@@ -764,6 +739,222 @@ static void parse_holtek_eeprom(unsigned short *ee_data)
 		   ee_data[6], ee_data[4], ee_data[5], ee_data[3]>>8, ee_data[3]&0xff);
 	return;
 }
+
+static const char *const winbond_xcvr[4] =
+{ "10baseT", "10base2", "AUI",
+  "10baseT extended length / reduced squelch level",};
+static const char *const winbond_rom[8] =
+{ "No", "No", "8KB", "16KB", "32KB", "64KB", "128KB", "256KB" };
+
+static const char winbond_bogus_msg[] = "\n\
+This board has bogus information written into the configuration EEPROM.\n\
+The configuration table is for a Realtek chip, but this board is using\n\
+a Winbond 89c940 chip.  This misprogramming usually results in an invalid\n\
+station address, and prevents the board from being detected by standard\n\
+drivers.\n\
+\n\
+To reprogram the configuration EEPROM select a new station address such as\n\
+40:00:00:12:34:56 and run this diagnostic with the options\n\
+    --write-EEPROM --emergency-rewrite --new-hwaddr 40:00:00:12:34:56\n\
+";
+
+static void parse_winbond_eeprom(unsigned short *ee_data)
+{
+	unsigned char *p = (void *)ee_data;
+	int i;
+
+	printf("Parsing the EEPROM of a Winbond PCI NE2000:\n");
+	printf("  Station Address ");
+	for (i = 0; i < 5; i++)
+		printf("%2.2X:", p[i]);
+	printf("%2.2X (used as the ethernet address).\n", p[i]);
+	printf(" PCI ID Vendor %#4.4x device %#4.4x  revision %d.\n"
+		   "   PCI maximum latency %d  minimum grant %d.\n"
+		   "   System control 0x%2.2x:\n"
+		   "     %s transceiver, %s\n"
+		   "     %s boot ROM",
+		   ee_data[9], ee_data[10], p[22], p[17], p[16], p[23],
+		   winbond_xcvr[p[23] & 3],
+		   p[23] & 8 ? "no link beat required" : "autoselect by link beat",
+		   winbond_rom[p[23]>>5]);
+	if (ee_data[9] ==  0x8c4a)
+		printf("%s", winbond_bogus_msg);
+			   
+	return;
+}
+
+static char unknown_eeprom_message[] = "\n\
+ No information is known about how to read or write the configuration\n\
+ EEPROM of this NIC board.  It is likely that this NIC is using a common\n\
+ chip with a unique PCI ID.  It may be possible to read the EEPROM by\n\
+ explicitly specifying the chip type and I/O address:\n\
+   ne2k-pci-diag -t 1 -p <ioaddr>	# For Realtek rtl8029 chip\n\
+   ne2k-pci-diag -t 2 -p <ioaddr>	# For Winbond 89C940 chip\n\
+   ne2k-pci-diag -t 3 -p <ioaddr>	# For Holtek HT80229 chip\n\
+ If one of these is required, please send a report to becker@scyld.com.\n\
+";
+static char no_rewrite_msg[] = "\n\
+*** Emergency EEPROM rewrite is not yet available for this chip type.\n\
+ ** See the Scyld diagnostic web page for updates and more information.\n\
+ ** http://www.scyld.com/diag/index.html\n";
+
+/* The examples below have 00:11:22:33:44:55 as the station address. */
+static unsigned short winbond_default_eeprom[] = {
+	0x1100, 0x3322, 0x5544, 0x0000, 0x0000, 0x0000, 0x0000, 0x5757,
+	0x0000, 0x1050, 0x0940, 0x0000, };
+static unsigned short via_default_eeprom[] = {
+	0x1100, 0x3322, 0x5544, 0x0000,	/* [3] == checksum+boardID */
+	0x0000, 0x0000, 0x0000, 0x0123,	/* [7] = subvendor ID */
+	0x5742, 0x0000, 0x0000, 0x0000,
+	0x0000, 0x0000, 0xBBAA, 0x73CC}; /* Config AA BB and CC, 0x73 locks. */
+
+
+/* Read, display, and possibly change the configuration EEPROM. */
+static void do_ne2k_eeprom(long ioaddr, int part_idx)
+{
+	int capabilities = pcidev_tbl[part_idx].flags;
+	int ee_addr_len = 6;		/* No known boards use a larger EEPROM. */
+	int i;
+
+	/* Read the EEPROM. */
+	if (capabilities & (RTL_EE | HOLTEK_EE | WINBOND_EE)) {
+		if (capabilities & (RTL_EE | WINBOND_EE))
+			outb(E8390_PAGE3, ioaddr + E8390_CMD);
+		for (i = 0; i < EEPROM_SIZE; i++)
+			eeprom_contents[i] =
+				read_eeprom(capabilities, ioaddr, i, ee_addr_len);
+	} else if (capabilities & VIA_EE) {
+		/* This may not be true, but we don't have an unprogrammed board
+		   to confirm functionality.
+		   Contact me (becker@scyld.com) if you want to work with me to
+		   get the EEPROM operations working. */
+		printf("The VIA Amazon EEPROM is locked."
+			   "  It cannot be read or updated.\n");
+		return;
+	} else {
+		if (show_eeprom)
+			printf(unknown_eeprom_message);
+		return;
+	}
+
+	if (emergency_rewrite  &&  ! (capabilities & (WINBOND_EE)))
+		fprintf(stderr, no_rewrite_msg);
+	if (emergency_rewrite  &&  ! set_hwaddr)
+		fprintf(stderr, "*** Emergency EEPROM rewrite is only valid when you"
+				" also specify a new\n"
+				"*** station address with -H <xx:xx:...>\n");
+
+	if (set_hwaddr) {
+		int eeprom_sa_offset = capabilities & RTL_EE ? 1 : 0;
+		const char *const field_names[] = {
+			"MAC address 0/1", "MAC address 2/3", "MAC address 4/5", };
+		memcpy(new_ee_contents, eeprom_contents, sizeof(eeprom_contents));
+		if (emergency_rewrite) {
+			if (capabilities & WINBOND_EE)
+				memcpy(new_ee_contents, winbond_default_eeprom,
+					   sizeof winbond_default_eeprom);
+			else if (capabilities & VIA_EE)
+				memcpy(new_ee_contents, via_default_eeprom,
+					   sizeof via_default_eeprom);
+		}
+
+		for (i = 0; i < 3; i++) {
+			new_ee_contents[i + eeprom_sa_offset] =
+				new_hwaddr[i*2] + (new_hwaddr[i*2+1]<<8);
+		}
+		if (emergency_rewrite && (capabilities & (WINBOND_EE))) {
+			printf("Writing new configuration EEPROM contents.\n");
+			for (i = 0; i < EEPROM_SIZE; i++)
+				if (new_ee_contents[i] != eeprom_contents[i])
+					do_update(capabilities, ioaddr, i, ee_addr_len, "EEPROM",
+							  eeprom_contents[i], new_ee_contents[i]);
+		} else {
+			printf("Writing new MAC station address.\n");
+			for (i = eeprom_sa_offset; i < eeprom_sa_offset + 3; i++)
+				if (new_ee_contents[i] != eeprom_contents[i])
+					do_update(capabilities, ioaddr, i, ee_addr_len,
+							  field_names[i - eeprom_sa_offset],
+							  eeprom_contents[i], new_ee_contents[i]);
+		}
+		/* Re-read all contents. */
+		for (i = 0; i < EEPROM_SIZE; i++)
+			eeprom_contents[i] =
+				read_eeprom(capabilities, ioaddr, i, ee_addr_len);
+	}
+
+#if ! defined(ENABLE_EEPROM_UPDATE)
+	if (new_default_media >= 0) {
+		printf("This release of the diagnostic does not include the code to\n"
+			   "write the default transceiver setting.\n");
+	}
+#else
+	if ((new_default_media & 0xff) > 4) {
+		fprintf(stderr "Media type %s (index %d) is invalid for this "
+				"board.\n", (new_default_media > 10) ? "<INVALID>" :
+				mediamap[new_default_media & 31].name, new_default_media);
+		
+	} else if (new_default_media >= 0) {
+		int offset = -1;
+		int word;
+		if (capability & WINBOND_EE) {
+			offset = 12;
+			word = eeprom_contents[offset] & ~0x0300;
+			word |= (new_default_media & 3)<<8;
+		} else if (capability & RTL_EE) {
+			offset = 0;
+			word = eeprom_contents[offset] & ~0x40C0;
+			/* Full duplex:0x4000. 10base2:0x00C0  10base5:0x0080*/
+			word |= (new_default_media == 0) ? 0x00 : 0xC0;
+		} else if (capability & HOLTEK_EE) {
+			offset = 3;
+			word = eeprom_contents[offset] & ~0x8f00;
+			/* 0x0400 Link beat required, 0x0800 polarity correction. */
+			word |= (new_default_media == 0) ? 0x0C00 : 0x0E00;
+		}
+		do_update(capabilities, ioaddr, offset, ee_addr_len,
+				  "Transceiver setting",
+				  eeprom_contents[offset], word);
+	}
+#endif
+
+	if (show_eeprom) {
+		int eeprom_words = EEPROM_SIZE;		/* Really  2<<ee_addr_len. */
+		int sum = 0;
+
+		printf("EEPROM contents (%d words):", eeprom_words);
+		for (i = 0; i < eeprom_words; i += 8) {
+			int j;
+			printf("\n0x%2.2x: ", i);
+			for (j = 0; j < 8; j++)
+				printf(" %4.4x", eeprom_contents[i + j]);
+			if (show_eeprom) {			/* Alt: show_eeprom > 1 */
+				printf("  ");
+				for (j = 0; j < 8; j++) {
+					int ew = eeprom_contents[i + j];
+					printf("%c%c",
+						   isprint(ew & 0xff) ? ew & 0xff : '_',
+						   isprint(ew >>   8) ? ew >> 8   : '_' );
+				}
+			}
+		}
+		for (i = 0; i < EEPROM_SIZE; i++)
+			sum += eeprom_contents[i];
+		printf("\n The word-wide EEPROM checksum is %#4.4x.\n", sum);
+	}
+
+	/* The user will usually want to see the interpreted EEPROM contents. */
+	if (verbose || show_eeprom) {
+		if (capabilities & RTL_EE)
+			parse_8029_eeprom(eeprom_contents);
+		else if (capabilities & HOLTEK_EE)
+			parse_holtek_eeprom(eeprom_contents);
+		else if (capabilities & WINBOND_EE)
+			parse_winbond_eeprom(eeprom_contents);
+	}
+
+	return;
+}
+
 
 /*
  * Local variables:
