@@ -14,24 +14,65 @@
 
 	The author may be reached as becker@scyld.com, or C/O
 	 Scyld Computing Corporation
-	 410 Severn Ave., Suite 210
+	 914 Bay Ridge Road, Suite 220
 	 Annapolis MD 21403
 
-	Support and updates available at
+	Updates and additional information are available at
 	http://www.scyld.com/diag/index.html
 	http://scyld.com/expert/mii-status.html
 	http://scyld.com/expert/NWay.html
 
 	Common-sense licensing statement: Using any portion of this program in
 	your own program means that you must give credit to the original author
-	and release the resulting code under the GPL.
+	and release the resulting code under the GPL.  To use this code under
+	other terms requires an explicit license from the copyright holder.
 */
 
 static char *version_msg =
-"pcnet-diag.c:v2.02 3/31/2001 Donald Becker (becker@scyld.com)\n"
+"pcnet-diag.c:v2.03 2/28/2005 Donald Becker (becker@scyld.com)\n"
 " http://www.scyld.com/diag/index.html\n";
 static char *usage_msg =
-"Usage: pcnet-diag [-aEefFmqrRtvVwW] [-p <IOport>] [-[AF] <media>]\n";
+"Usage: pcnet-diag [-aDEefFGhmqrRtvVwW] [-p <IOport>] [-[AF] <media>]\n"
+"  For details and other options see http://www.scyld.com/diag/index.html\n";
+
+static const char long_usage_msg[] =
+"Usage: %s [-aDfrRvVw] [-AF <speed+duplex>] [-#<BoardNum>]\n\
+\n\
+   Show the internal state of a network adapter.\n\
+\n\
+   The common usage is\n\
+      diag -aem\n\
+\n\
+ Frequently used options are\n\
+   -a  --show_all_registers	Print all registers.\n\
+   -e  --show-eeprom	Dump EEPROM contents, \"-ee\" shows the details.\n\
+   -m  --show_mii		Print the MII transceiver state\n\
+					Using -mm monitors the link.\n\
+   -f  --force		Perform operation, even on a running NIC.\n\
+\n\
+ To operate on a single NIC, or one that hasn't been automatically found:\n\
+   -#  --card_num	INDEX	Operate on the specified card index.\n\
+   -p  --port-base	IOADDR	Assume an adapter at the specified I/O address.\n\
+   -t  --chip-type	TYPE	Specify adapter type (with '-p').  Use '-1' to\n\
+                            list available types indicies.\n\
+\n\
+ To change the persistent EEPROM settings\n\
+   -F, --new-interface N	Set the default transceiver type.\n\
+   -G  --parameters PARMS	Set adapter-specific parameters.\n\
+   -H  --new-hwaddr 01:23:45:67:89:ab\n\
+        Set a new hardware station address.  Typically disabled for safety.\n\
+   -w  --write-EEPROM	 Actually write the new settings into the EEPROM.\n\
+ To read and write the boot BIOS extension Flash ROM\n\
+   -B       Show the first few bytes of the ROM\n\
+   -L FILE  Load the Flash from FILE.\n\
+   -S FILE  Store the Flash image to FILE.\n\
+\n\
+   -D  --debug\n\
+   -v  --verbose	Report each action taken.\n\
+   -V  --version	Emit version information.\n\
+\n\
+   -A  --advertise <speed|setting>  (See the mii-diag manual page.)\n\
+\n";
 
 #if ! defined(__OPTIMIZE__)
 #warning  You must compile this program with the correct options!
@@ -55,7 +96,7 @@ static char *usage_msg =
 #include <sys/io.h>
 #endif
 
-/* No libmii.h or libflash.h yet. */
+/* No libmii.h or libflash.h yet, thus the declarations here. */
 extern int show_mii_details(long ioaddr, int phy_id);
 extern int monitor_mii(long ioaddr, int phy_id);
 
@@ -66,17 +107,18 @@ extern int (*flash_in_hook)(long addr, int offset);
 extern void (*flash_out_hook)(long addr, int offset, int val);
 
 /* We should use __u8 .. __u32, but they are not always defined. */
-typedef unsigned int u32;
-typedef unsigned short u16;
-typedef unsigned char u8;
+typedef u_int32_t u32;
+typedef u_int16_t u16;
+typedef u_int8_t u8;
 
 struct option longopts[] = {
  /* { name  has_arg  *flag  val } */
+	{"card-num", 1, 0, '#'},		/* Operate on the specified card index. */
 	{"Advertise", 1, 0, 'A'},
 	{"base-address", 1, 0, 'p'},
 	{"show_all_registers",	0, 0, 'a'},	/* Print all registers. */
-	{"help",	0, 0, 'h'},	/* Give help */
-	{"show-eeprom",  0, 0, 'e'}, /* Dump EEPROM contents (-ee valid). */
+	{"help",	0, 0, 'h'},			/* Print a long usage message. */
+	{"show-eeprom",  0, 0, 'e'},	/* Dump EEPROM contents (-ee valid). */
 	{"emergency-rewrite",  0, 0, 'E'}, /* Re-write a corrupted EEPROM.  */
 	{"force-detection",  0, 0, 'f'},
 	{"new-interface",  1, 0, 'F'},	/* New interface (built-in, AUI, etc.) */
@@ -84,7 +126,7 @@ struct option longopts[] = {
 	{"show-mii",  0, 0, 'm'},	/* Dump MII management registers. */
 	{"port-base",  1, 0, 'p'},	/* Use the specified I/O address. */
 	{"quiet",	0, 0, 'q'},		/* Decrease verbosity */
-	{"Reset",	0, 0, 'R'},		/* Reset the transceiver. */
+	{"reset",	0, 0, 'R'},		/* Reset the transceiver. */
 	{"chip-type",  1, 0, 't'},	/* Assume the specified chip type index. */
 	{"test",	0, 0, 'T'},		/* Do register and SRAM test. */
 	{"verbose",	0, 0, 'v'},		/* Verbose mode */
@@ -99,7 +141,7 @@ enum pcnet_flags { PCNET_HAS_MII };
 
 /* The table of known chips.
    Because of the bogus /proc/pci interface we must have both the exact
-   name and a PCI vendor/device IDs.
+   name from the kernel, a common name and the PCI vendor/device IDs.
    This table is searched in order: place specific entries followed by
    'catch-all' general entries. */
 struct pcidev_entry {
@@ -135,6 +177,7 @@ static unsigned int opt_flash_show = 0;
 static char	*opt_flash_dumpfile = NULL, *opt_flash_loadfile = NULL;
 
 static unsigned char new_hwaddr[6], set_hwaddr = 0;
+static int emergency_rewrite = 0;
 
 static int scan_proc_pci(int card_num);
 static int parse_media_type(const char *capabilities);
@@ -146,12 +189,11 @@ main(int argc, char **argv)
 {
 	int port_base = 0, chip_type = 0;
 	int errflag = 0, show_version = 0;
-	int emergency_rewrite = 0;
 	int c, longind;
 	int card_num = 0;
 	extern char *optarg;
 
-	while ((c = getopt_long(argc, argv, "#:aA:DeEfF:G:mp:qrRst:vVwWH:BL:S:",
+	while ((c = getopt_long(argc, argv, "#:aA:DeEfF:G:hH:mp:qrRst:vVwWBL:S:",
 							longopts, &longind))
 		   != -1)
 		switch (c) {
@@ -166,7 +208,8 @@ main(int argc, char **argv)
 			if (new_default_media < 0)
 				errflag++;
 			break;
-		case 'G': opt_G++; opt_GPIO = strtol(optarg, NULL, 16); break;
+		case 'G': opt_G++; opt_GPIO = strtoul(optarg, NULL, 16); break;
+		case 'h': fprintf(stderr, long_usage_msg, argv[0]); return 0;
 		case 'H':
 			{
 				int hwaddr[6], i;
@@ -182,7 +225,7 @@ main(int argc, char **argv)
 			}
 		case 'm': show_mii++;	 break;
 		case 'p':
-			port_base = strtol(optarg, NULL, 16);
+			port_base = strtoul(optarg, NULL, 16);
 			break;
 		case 'q': if (verbose) verbose--;		 break;
 		case 'r': opt_restart++;	break;
@@ -279,8 +322,9 @@ static int scan_proc_bus_pci(int card_num)
 				   &pci_bus, &pci_devid, &irq, &pciaddr0, &pciaddr1) <= 0)
 			break;
 		for (i = 0; pcidev_tbl[i].vendor; i++) {
-			if ((pci_devid & (pcidev_tbl[i].device_mask | 0xffff0000)) !=
-				(pcidev_tbl[i].vendor << 16) + pcidev_tbl[i].device)
+			if ((pci_devid >> 16) != pcidev_tbl[i].vendor
+				|| (pci_devid & pcidev_tbl[i].device_mask) !=
+				pcidev_tbl[i].device)
 				continue;
 			chip_idx = i;
 			card_cnt++;
@@ -396,6 +440,7 @@ static int parse_media_type(const char *capabilities)
 		"100baseT4", "100baseTx", "100baseTx-FD", "100baseTx-HD",
 		"10baseT", "10baseT-FD", "10baseT-HD", 0,
 	};
+	char *endptr;
 	int cap_map[] = { 0x0200, 0x0180, 0x0100, 0x0080, 0x0060, 0x0040, 0x0020,};
 	int i;
 	if (debug)
@@ -403,7 +448,8 @@ static int parse_media_type(const char *capabilities)
 	for (i = 0; mtypes[i]; i++)
 		if (strcasecmp(mtypes[i], capabilities) == 0)
 			return cap_map[i];
-	if ((i = strtol(capabilities, NULL, 16)) <= 0xffff)
+	i = strtoul(capabilities, &endptr, 16);
+	if (*endptr == 0  &&  0 < i  &&  i <= 0xffff)
 		return i;
 	fprintf(stderr, "Invalid media advertisement '%s'.\n", capabilities);
 	return 0;
@@ -435,16 +481,20 @@ struct { char *name; int value; } mediamap[] = {
 
 static int get_media_index(const char *name)
 {
+	char *endptr;
 	int i;
+
+	if (! name)
+		return -1;
 	for (i = 0; mediamap[i].name; i++)
 		if (strcasecmp(name, mediamap[i].name) == 0)
 			return i;
-	if (name  &&  atoi(name) >= 00)
-		return atoi(name);
-	fprintf(stderr, "Invalid interface specified: it must be one of\n  ");
+	i = strtol(name, &endptr, 0);
+	if (*endptr == 0)
+		return i;
+	fprintf(stderr, "Invalid interface specified.  It must be one of\n");
 	for (i = 0; mediamap[i].name; i++)
-		fprintf(stderr, "  %s", mediamap[i].name);
-	fprintf(stderr, ".\n");
+		fprintf(stderr, "  %d %s\n", mediamap[i].value, mediamap[i].name);
 	return -1;
 }
 
