@@ -13,7 +13,7 @@
 
 	The author may be reached as becker@scyld.com, or C/O
 	 Scyld Computing Corporation
-	 410 Severn Ave., Suite 210
+	 914 Bay Ridge Road, Suite 220
 	 Annapolis MD 21403
 
 	References
@@ -29,7 +29,7 @@
 */
 
 static char *version_msg =
-"rtl8139-diag.c:v2.11 4/22/2003 Donald Becker (becker@scyld.com)\n"
+"rtl8139-diag.c:v2.13 2/28/2005 Donald Becker (becker@scyld.com)\n"
 " http://www.scyld.com/diag/index.html\n";
 static char *usage_msg =
 "Usage: rtl8139-diag [-aDEefFGhmqrRtvVwW] [-p <IOport>] [-[AF] <media>]\n"
@@ -160,10 +160,22 @@ struct pcidev_entry {
 	  0x1113, 0x1211, 0xffff, 0, 128, rtl81x9_diag },
 	{ "D-Link DFE-530TX+ (RealTek RTL8139C)", 0,
 	  0x1186, 0x1300, 0xffff, 0, 256, rtl81x9_diag },
+	{ "D-Link DFE-690TXD CardBus (RealTek RTL8139C)", 0,
+	  0x1186, 0x1340, 0xffff, 0, 256, rtl81x9_diag },
+	{ "D-Link NIC (Unknown type, assuming realtek RTL8139C)", 0,
+	  0x1186, 0x1300, 0xff00, 0, 256, rtl81x9_diag },
 	{ "RealTek (unknown chip type)", 0,
 	  0x10ec, 0x8100, 0xff00, 0, 128, rtl81x9_diag },
+	{ "LevelOne FPC-0106Tx (RTL8139C)", 0,
+	  0x018a, 0x1060, 0xffff, 0, 128, rtl81x9_diag },
 	{ "Compaq HNE-300 (RTL8139C)", 0,
 	  0x021b, 0x8139, 0xffff, 0, 128, rtl81x9_diag },
+	{ "Edimax EP-4103DL CardBus (RTL8139C)", 0,
+	  0x13d1, 0xab06, 0xffff, 0, 128, rtl81x9_diag },
+	{ "Siemens SpeedStream 1012v2 CardBus (RTL8139C)", 0,
+	  0x02ac, 0x1012, 0xffff, 0, 128, rtl81x9_diag },
+	{ "Siemens SpeedStream 1020 (RTL8139C)", 0,
+	  0x1432, 0x9130, 0xffff, 0, 128, rtl81x9_diag },
 	{ 0, 0, 0, 0},
 };
 
@@ -887,9 +899,18 @@ static int rtl81x9_eeprom(long ioaddr, int flags)
 			0x8129, 0x10ec, 0x8139, 0x10ec, 0x8139, 0x4020, 0xe110, 0x2301,
 			0x6745, 0xab89, 0x4d10, 0xf7c2, 0x8001, 0xb388, 0x58fa, 0x0708,
 			0xd843, 0xa438, 0xd843, 0xa438, 0xd843, 0xa438, 0xd843, 0xa438, };
-		if (emergency_rewrite)
+		/* A rtl8139c table is
+		 * 0x00:  8129 10ec 8139 10ec 8139 4020 e112 5000
+		 * 0x08:  b122 6e2a 4d14 f7c2 8801 43b9 b0f2 071a
+		 * 0x10:  df43 8a36 df43 8a36 43b9 b0f2 1111 1111
+		 */
+		if (emergency_rewrite > 2)
 			memcpy(new_ee_contents, backup_ee_contents,
 				   sizeof backup_ee_contents);
+		else if (emergency_rewrite)
+			fprintf(stderr, "Overwriting the configuration EEPROM is likely "
+					"to do more harm than good.\n  Pass -EEE if you are "
+					"certain this is what you want to do.");
 		for (i = 0; i < 3; i++)
 			new_ee_contents[i + EEPROM_SA_OFFSET] =
 				(new_hwaddr[i*2+1]<<8) + new_hwaddr[i*2];
