@@ -16,7 +16,7 @@
 
 	The author may be reached as becker@scyld.com, or C/O
 	 Scyld Computing Corporation
-	 410 Severn Ave., Suite 210
+	 914 Bay Ridge Road, Suite 220
 	 Annapolis MD 21403
 
 	Support and updates available at
@@ -31,49 +31,49 @@
 */
 
 static char *version_msg =
-"tulip-diag.c:v2.17 5/6/2003 Donald Becker (becker@scyld.com)\n"
+"tulip-diag.c:v2.18 11/12/2003 Donald Becker (becker@scyld.com)\n"
 " http://www.scyld.com/diag/index.html\n";
 static char *usage_msg =
 "Usage: tulip-diag [-aDEefFGhmqrRtvVwW] [-p <IOport>] [-[AF] <media>]\n"
 "  For details and other options see http://www.scyld.com/diag/index.html\n";
 
 static const char long_usage_msg[] =
-"Usage: %s [-aDfrRvVw] [-AF <speed+duplex>] [-#<BoardNum>]
-
-   Show the internal state of a network adapter.
-
-   The common usage is
-      diag -aem
-
- Frequently used options are
-   -a  --show_all_registers	Print all registers.
-   -e  --show-eeprom	Dump EEPROM contents, \"-ee\" shows the details.
-   -m  --show_mii		Print the MII transceiver state
-					Using -mm monitors the link.
-   -f  --force		Perform operation, even on a running NIC.
-
- To operate on a single NIC, or one that hasn't been automatically found:
-   -#  --card_num	INDEX	Operate on the specified card index.
-   -p  --port-base	IOADDR	Assume an adapter at the specified I/O address.
-   -t  --chip-type	TYPE	Specify adapter type (with '-p'), use '-1' to list.
-
- To change the persistent EEPROM settings
-   -F, --new-interface N	Set the default transceiver type.
-   -G  --parameters PARMS	Set adapter-specific parameters.
-   -H  --new-hwaddr 01:23:45:67:89:ab
-        Set a new hardware station address.  Typically disabled for safety.
-   -w  --write-EEPROM	 Actually write the new settings into the EEPROM.
- To read and write the boot BIOS extension Flash ROM
-   -B       Show the first few bytes of the ROM
-   -L FILE  Load the Flash from FILE.
-   -S FILE  Store the Flash image to FILE.
-
-   -D  --debug
-   -v  --verbose	Report each action taken.
-   -V  --version	Emit version information.
-
-   -A  --advertise <speed|setting>  (See the mii-diag manual page.)
-";
+"Usage: %s [-aDfrRvVw] [-AF <speed+duplex>] [-#<BoardNum>]\n"
+"\n"
+"   Show the internal state of a network adapter.\n"
+"\n"
+"   The common usage is\n"
+"      diag -aem\n"
+"\n"
+" Frequently used options are\n"
+"   -a  --show_all_registers	Print all registers.\n"
+"   -e  --show-eeprom	Dump EEPROM contents, \"-ee\" shows the details.\n"
+"   -m  --show_mii		Print the MII transceiver state\n"
+"					Using -mm monitors the link.\n"
+"   -f  --force		Perform operation, even on a running NIC.\n"
+"\n"
+" To operate on a single NIC, or one that hasn't been automatically found:\n"
+"   -#  --card_num	INDEX	Operate on the specified card index.\n"
+"   -p  --port-base	IOADDR	Assume an adapter at the specified I/O address.\n"
+"   -t  --chip-type	TYPE	Specify adapter type (with '-p'), use '-1' to list.\n"
+"\n"
+" To change the persistent EEPROM settings\n"
+"   -F, --new-interface N	Set the default transceiver type.\n"
+"   -G  --parameters PARMS	Set adapter-specific parameters.\n"
+"   -H  --new-hwaddr 01:23:45:67:89:ab\n"
+"        Set a new hardware station address.  Typically disabled for safety.\n"
+"   -w  --write-EEPROM	 Actually write the new settings into the EEPROM.\n"
+" To read and write the boot BIOS extension Flash ROM\n"
+"   -B       Show the first few bytes of the ROM\n"
+"   -L FILE  Load the Flash from FILE.\n"
+"   -S FILE  Store the Flash image to FILE.\n"
+"\n"
+"   -D  --debug\n"
+"   -v  --verbose	Report each action taken.\n"
+"   -V  --version	Emit version information.\n"
+"\n"
+"   -A  --advertise <speed|setting>  (See the mii-diag manual page.)\n"
+"\n";
 
 #if ! defined(__OPTIMIZE__)
 #warning  You must compile this program with the correct options!
@@ -223,6 +223,8 @@ struct pcidev_entry {
 	0x14f1, 0x1803, 0xffff, IS_CONEXANT, 256, tulip_diag },
   { "3Com 3cSOHO100B-TX (ADMtek Centaur)", 0,
 	0x10b7, 0x9300, 0xffff, COMET_MACADDR, 256, tulip_diag },
+  { "3Com 3cSOHO100B-TX (ADMtek Centaur ID 8c00)", 0,
+	0x10b7, 0x8c00, 0xffff, COMET_MACADDR, 256, tulip_diag },
 	{ 0, 0, 0, 0},
 };
 
@@ -880,6 +882,7 @@ int tulip_diag(int vendor_id, int device_id, long ioaddr, int part_idx)
 static void tulip_eeprom(long ioaddr, int flags)
 {
 	int eeprom_words = 64;
+	int eeprom_change = 0;
 	int i;
 
 	/* Read the EEPROM. */
@@ -936,37 +939,85 @@ static void tulip_eeprom(long ioaddr, int flags)
 			davicom_eeprom((unsigned char *)eeprom_contents, flags);
 		else
 			parse_eeprom(ioaddr, (unsigned char *)eeprom_contents, flags);
-
-		if (show_eeprom > 1) {
-			int block_crc = (calculate_checksum1(eeprom_contents, 8)>>8) & 0xff;
-			int full_crc =
-				(ether_crc_le((void*)eeprom_contents, 126) ^ 0xffff) & 0xffff;
-			printf("EEPROM contents (%d words):", eeprom_words);
-			for (i = 0; i < eeprom_words; i += 8) {
-				int j;
-				printf("\n0x%2.2x: ", i);
-				for (j = 0; j < 8; j++)
-					printf(" %4.4x", eeprom_contents[i + j]);
-				if (show_eeprom > 2) {
-					printf("  ");
-					for (j = 0; j < 8; j++) {
-						int ew = eeprom_contents[i + j];
-						printf("%c%c",
-							   isprint(ew & 0xff) ? ew & 0xff : '_',
-							   isprint(ew >>   8) ? ew >> 8   : '_' );
-					}
-				}
-			}
-			printf("\n ID block CRC %#2.2x (vs. %#2.2x).\n"
-				   "  Full contents CRC 0x%4.4x (read as 0x%4.4x).\n",
-				   block_crc, eeprom_contents[8] & 0xff,
-				   full_crc, eeprom_contents[63]);
-		}
 	}
+
+	if (show_eeprom > 1) {
+		int block_crc = (calculate_checksum1(eeprom_contents, 8)>>8) & 0xff;
+		int full_crc =
+			(ether_crc_le((void*)eeprom_contents, 126) ^ 0xffff) & 0xffff;
+		printf("EEPROM contents (%d words):", eeprom_words);
+		for (i = 0; i < eeprom_words; i += 8) {
+			int j;
+			printf("\n0x%2.2x: ", i);
+			for (j = 0; j < 8; j++)
+				printf(" %4.4x", eeprom_contents[i + j]);
+			printf("  ");
+			for (j = 0; j < 8; j++) {
+				int ew = eeprom_contents[i + j];
+				printf("%c%c", isprint(ew & 0xff) ? ew & 0xff : '_',
+					   isprint(ew >>   8) ? ew >> 8   : '_' );
+			}
+		}
+
+		printf("\n ID block CRC %#2.2x (vs. %#2.2x).\n"
+			   "  Full contents CRC 0x%4.4x (read as 0x%4.4x).\n",
+			   block_crc, eeprom_contents[8] & 0xff,
+			   full_crc, eeprom_contents[63]);
+	}
+
 	/* Check for a bogus Intel CardBus card. */
 	if (eeprom_contents[0] == 0x8086  &&  eeprom_contents[1] == 0x0001  &&
 		eeprom_contents[2] == 0x0087)
 		check_for_intel_cb(ioaddr, eeprom_contents);
+
+#if 0
+	if (emergency_rewrite)
+		fprintf(stderr, "Configuration EEPROM overwrite is not available in "
+				"this version of the diagnostic.\n");
+#else
+	if (emergency_rewrite  &&  ! set_hwaddr)
+		printf("*** Emergency EEPROM rewrite is only valid when you also "
+			   "specify a new\n*** station address with -H <xx:xx:...>\n");
+#endif
+
+	if (set_hwaddr) {
+		if (flags & PNIC_MTABLE) {
+			/* int eeprom_sa_offset = 10; */
+			;
+		} else if (flags & COMET_MACADDR) {			/* All ADMtek chips. */
+			u16 backup_ee_contents[] = {
+				0x0985, 0x0002, 0x0000, 0x0000, 0x0001, 0xd618, 0x126f, 0x0000,
+				0x0000, 0x0400, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
+				0x0985, 0x1317, 0x1216, 0x1113, 0x00e8, 0x0202, 0x0000, 0x80cc,
+			};
+			int eeprom_sa_offset = 4;
+			if (emergency_rewrite)
+				memcpy(new_ee_contents, backup_ee_contents,
+					   sizeof backup_ee_contents);
+			for (i = 0; i < 3; i++)
+				new_ee_contents[i + eeprom_sa_offset] =
+					(new_hwaddr[i*2]<<8) + new_hwaddr[i*2+1];
+			eeprom_change++;
+		} else {
+			fprintf(stderr, "Configuration EEPROM writing is not supported "
+				"for this chip type.\n");
+		}
+	}
+
+	if (eeprom_change) {
+		/* Enable EEPROM write, if the chip requires anything special. */
+		if (do_write_eeprom) {
+			for (i = 0; i < eeprom_words; i++)
+				if (new_ee_contents[i] != eeprom_contents[i]) {
+					do_update(ioaddr, eeprom_contents, i,
+							  "configuration update", new_ee_contents[i]);
+				}
+		} else {
+			printf("Would write %4.4x to replace %4.4x at %d.\n",
+				   new_ee_contents[i], eeprom_contents[i], i);
+		}
+	}
+
 	if (new_default_media >= 0) {
 		if (default_media_offset > 0)
 			do_update(ioaddr, eeprom_contents, default_media_offset/2,
@@ -1410,10 +1461,10 @@ static void parse_eeprom(long ioaddr, unsigned char *ee_data, int flags)
 	  /* Should actually do a fix-up based on the vendor half of the station
 		 address prefix here.  Or least use that information to report which
 		 transceiver will work. */
-		printf("  * An old-style EEPROM layout was found.
-  * The old-style layout does not contain transceiver control information.
-  * This board may not work, or may work only with a subset of transceiver
-  * options or data rates.\n");
+		printf("  * An old-style EEPROM layout was found.\n"
+"  * The old-style layout does not contain transceiver control information.\n"
+"  * This board may not work, or may work only with a subset of transceiver\n"
+"  * options or data rates.\n");
 	  return;
 	} else if (ee_data[27] == 0) {
 		printf(" A simplifed EEPROM data table was found.\n"
@@ -1696,14 +1747,12 @@ static void davicom_eeprom(unsigned char *ee_data, int part_idx)
 }
 
 static const char msg_no_station_addr[] =
-"  However this card's station address could not be located in Card
- Information Structure, therefore no corrected EEPROM table information can
- be written.
-";
+"  However this card's station address could not be located in Card\n"
+" Information Structure, therefore no corrected EEPROM table information can\n"
+" be written.\n";
 
 static const char msg_bogus_intel_eeprom[] =
-"You have an Intel CardBus card with an incomplete EEPROM!
-";
+"You have an Intel CardBus card with an incomplete EEPROM!\n";
 
 static void check_for_intel_cb(long ioaddr, u_int16_t *eeprom_contents)
 {
