@@ -4,7 +4,7 @@
    based on the following chips:
 	VIA Rhine vt86c100, vt3043, vt6102, and vt6105 Ethernet controllers.
 
-	Copyright 1998-2002 by Donald Becker.
+	Copyright 1998-2004 by Donald Becker.
 	This software may be used and distributed according to the terms of
 	the GNU General Public License (GPL), incorporated herein by reference.
 	Contact the author for use under other terms.
@@ -14,10 +14,10 @@
 
 	The author may be reached as becker@scyld.com, or C/O
 	 Scyld Computing Corporation
-	 410 Severn Ave., Suite 210
+	 914 Bay Ridge Road, Suite 220
 	 Annapolis MD 21403
 
-	Support and updates available at
+	Updates and additional information available at
 	http://www.scyld.com/diag/index.html
 	http://scyld.com/expert/mii-status.html
 	http://scyld.com/expert/NWay.html
@@ -27,52 +27,53 @@
 
 	Common-sense licensing statement: Using any portion of this program in
 	your own program means that you must give credit to the original author
-	and release the resulting code under the GPL.
+	and release the resulting code under the GPL.  To use this code under
+	other terms requires an explicit license from the copyright holder.
 */
 
 static char *version_msg =
-"via-diag.c:v2.09 12/28/2002 Donald Becker (becker@scyld.com)\n"
+"via-diag.c:v2.10 2/28/2005 Donald Becker (becker@scyld.com)\n"
 " http://www.scyld.com/diag/index.html\n";
 static char *usage_msg =
-"Usage: via-diag [-aEefFGhmqrRtvVwW] [-p <IOport>] [-[AF] <media>]\n"
+"Usage: via-diag [-aDEefFGhmqrRtvVwW] [-p <IOport>] [-[AF] <media>]\n"
 "  For details and other options see http://www.scyld.com/diag/index.html\n";
 
 static const char long_usage_msg[] =
-"Usage: %s [-aDfrRvVw] [-AF <speed+duplex>] [-#<BoardNum>]
-
-   Show the internal state of a network adapter.
-
-   The common usage is
-      diag -aem
-
- Frequently used options are
-   -a  --show_all_registers	Print all registers.
-   -e  --show-eeprom	Dump EEPROM contents, \"-ee\" shows the details.
-   -m  --show_mii		Print the MII transceiver state
-					Using -mm monitors the link.
-   -f  --force		Perform operation, even on a running NIC.
-
- To operate on a single NIC, or one that hasn't been automatically found:
-   -#  --card_num	INDEX	Operate on the specified card index.
-   -p  --port-base	IOADDR	Assume an adapter at the specified I/O address.
-   -t  --chip-type	TYPE	Specify adapter type with '-p', use '-1' to list.
-
- To change the persistent configuration EEPROM settings
-   -G  --parameters PARMS	Set adapter-specific parameters.
-   -H  --new-hwaddr 01:23:45:67:ab:cd
-			Set a new hardware station address.  Typically diabled, 
-   -w  --write-EEPROM	 Actually write the new settings into the EEPROM.
- To read and write the boot BIOS extension Flash ROM
-   -B       Show the first few bytes of the ROM
-   -L FILE  Load the Flash from FILE.
-   -S FILE  Store the Flash image to FILE.
-
-   -D  --debug
-   -v  --verbose	Report each action taken.
-   -V  --version	Emit version information.
-
-   -A  --advertise <speed|setting>  (See the mii-diag manual page.)
-";
+"Usage: %s [-aDfrRvVw] [-AF <speed+duplex>] [-#<BoardNum>]\n"
+"\n"
+"   Show the internal state of a network adapter.\n"
+"\n"
+"   The common usage is\n"
+"      diag -aem\n"
+"\n"
+" Frequently used options are\n"
+"   -a  --show_all_registers	Print all registers.\n"
+"   -e  --show-eeprom	Dump EEPROM contents, \"-ee\" shows the details.\n"
+"   -m  --show_mii		Print the MII transceiver state\n"
+"					Using -mm monitors the link.\n"
+"   -f  --force		Perform operation, even on a running NIC.\n"
+"\n"
+" To operate on a single NIC, or one that hasn't been automatically found:\n"
+"   -#  --card_num	INDEX	Operate on the specified card index.\n"
+"   -p  --port-base	IOADDR	Assume an adapter at the specified I/O address.\n"
+"   -t  --chip-type	TYPE	Specify adapter type (with '-p'), use '-1' to list.\n"
+"\n"
+" To change the persistent configuration EEPROM settings\n"
+"   -G  --parameters PARMS	Set adapter-specific parameters.\n"
+"   -H  --new-hwaddr 01:23:45:67:89:ab\n"
+"        Set a new hardware station address.  Typically disabled for safety.\n"
+"   -w  --write-EEPROM	 Actually write the new settings into the EEPROM.\n"
+" To read and write the boot BIOS extension Flash ROM\n"
+"   -B       Show the first few bytes of the ROM\n"
+"   -L FILE  Load the Flash from FILE.\n"
+"   -S FILE  Store the Flash image to FILE.\n"
+"\n"
+"   -D  --debug\n"
+"   -v  --verbose	Report each action taken.\n"
+"   -V  --version	Emit version information.\n"
+"\n"
+"   -A  --advertise <speed|setting>  (See the mii-diag manual page.)\n"
+"\n";
 
 #if ! defined(__OPTIMIZE__)
 #warning  You must compile this program with the correct options!
@@ -129,6 +130,7 @@ struct option longopts[] = {
 	{"show-mii",  0, 0, 'm'},	/* Dump MII management registers. */
 	{"port-base",  1, 0, 'p'},	/* Use the specified I/O address. */
 	{"quiet",	0, 0, 'q'},		/* Decrease verbosity */
+	{"restart",	0, 0, 'r'},		/* Restart autonegotiation. */
 	{"reset",	0, 0, 'R'},		/* Reset the transceiver. */
 	{"chip-type",  1, 0, 't'},	/* Assume the specified chip type index. */
 	{"test",	0, 0, 'T'},		/* Do register and SRAM test. */
@@ -557,9 +559,9 @@ static const char *intr_names[16] ={
 };
 
 /* Values read from the EEPROM, and a new image to write. */
-#define EEPROM_SIZE 256
-unsigned short eeprom_contents[EEPROM_SIZE];
-unsigned short new_ee_contents[EEPROM_SIZE];
+#define MAX_EEPROM_SIZE 256
+unsigned short eeprom_contents[MAX_EEPROM_SIZE];
+unsigned short new_ee_contents[MAX_EEPROM_SIZE];
 #define EEPROM_SA_OFFSET	0x00
 #define EEPROM_CSUM_OFFSET	0	/* 0 means none. */
 
@@ -636,14 +638,14 @@ int rhine_diag(int vendor_id, int device_id, long ioaddr, int part_idx)
 		unsigned intr_status;
 
 		if (chip_active && !opt_f) {
-			printf("This device appears to be active, so some registers"
+			printf(" This device appears to be active, so some registers"
 				   " will not be read.\n"
-				   "To see all register values use the '-f' flag.\n");
+				   " To see all register values use the '-f' flag.\n");
 		} else
 			chip_active = 0;		/* Ignore the chip status with -f */
 
 		if (opt_a) {
-			/* Reading some registers hoses the chip operation. */
+			/* Reading some registers disrupts chip operation. */
 			char dont_read[8] = {0x00,0x00,0x00,0xce,0xfd,0xed,0x7d,0xff};
 			printf("%s chip registers at %#lx",
 				   pcidev_tbl[part_idx].part_name, ioaddr);
@@ -667,7 +669,9 @@ int rhine_diag(int vendor_id, int device_id, long ioaddr, int part_idx)
 		}
 	}
 
-	if (show_eeprom) {		/* Add EEPROM sizing code here. */
+	if (show_eeprom || set_hwaddr || new_default_media >= 0) {
+		/* The Rhine has the questionable ability to 'lock' EEPROM access.
+		   EEPROM sizing cannot be done unless the EEPROM is unlocked. */
 		if (inb(ioaddr + EECtrl) & 0x80) {
 			printf("  Access to the EEPROM has been disabled (0x%2.2x).\n"
 				   "    Direct reading or writing is not possible.\n",
@@ -689,6 +693,16 @@ int rhine_diag(int vendor_id, int device_id, long ioaddr, int part_idx)
 		}
 	}
 
+#if defined(HAS_EEPROM_RECOVERY)
+	if (emergency_rewrite)
+		fprintf(stderr, "Full EEPROM overwrite is not available in this"
+				"version of the diagnostic.\n");
+#else
+	if (emergency_rewrite  &&  ! set_hwaddr)
+		printf("*** Emergency EEPROM rewrite is only valid when you also "
+			   "specify a new\n*** station address with -H <xx:xx:...>\n");
+#endif
+
 	if (set_hwaddr) {
 		memcpy(new_ee_contents, eeprom_contents, eeprom_size << 1);
 		for (i = 0; i < 3; i++)
