@@ -2,9 +2,9 @@
 
    This is a diagnostic and EEPROM setup program for Ethernet adapters
    based on the 3Com Vortex, Boomerang and Cyclone chips, as used on the
-   3Com 3c590/595/900/905 PCI EtherLink XL adapters.
+   3Com 3c590/595/900/905/920 PCI EtherLink XL adapters.
 
-	Copyright 1997-2003 by Donald Becker.
+	Copyright 1997-2004 by Donald Becker.
 	This software may be used and distributed according to the terms of
 	the GNU General Public License (GPL), incorporated herein by reference.
 	Contact the author for use under other terms.
@@ -14,10 +14,10 @@
 
 	The author may be reached as becker@scyld.com, or C/O
 	 Scyld Computing Corporation
-	 410 Severn Ave., Suite 210
+	 914 Bay Ridge Road, Suite 220
 	 Annapolis MD 21403
 
-	Updates and additional information are available at
+	Updates and additional information available at
 	http://www.scyld.com/diag/index.html
 	http://scyld.com/expert/mii-status.html
 	http://scyld.com/expert/NWay.html
@@ -34,48 +34,48 @@
 */
 
 static char *version_msg =
-"vortex-diag.c:v2.14 12/28/2002 Donald Becker (becker@scyld.com)\n"
+"vortex-diag.c:v2.16 1/12/2004 Donald Becker (becker@scyld.com)\n"
 " http://www.scyld.com/diag/index.html\n";
 static char *usage_msg =
 "Usage: vortex-diag [-aDEefFgGhmPqrRtvVwW] [-p <IOport>] [-[AF] <media>]\n"
 "  For details and other options see http://www.scyld.com/diag/index.html\n";
 
 static const char long_usage_msg[] =
-"Usage: %s [-aDfrRvVw] [-AF <speed+duplex>] [-#<BoardNum>]
-
-   Show the internal state of a network adapter.
-
-   The common usage is
-      vortex-diag -aem
-
- Frequently used options are
-   -a  --show_all_registers	Print all registers.
-   -e  --show-eeprom	Dump EEPROM contents, \"-ee\" shows the details.
-   -m  --show_mii		Print the MII transceiver state
-					Using -mm monitors the link.
-   -f  --force		Perform operation, even on a running NIC.
-
- To operate on a single NIC, or one that hasn't been automatically found:
-   -#  --card_num	INDEX	Operate on the specified card index.
-   -p  --port-base	IOADDR	Assume an adapter at the specified I/O address.
-   -t  --chip-type	TYPE	Specify adapter type with '-p', use '-1' to list.
-
- To change the persistent EEPROM settings
-   -G  --parameters PARMS	Set adapter-specific parameters.
-   -H  --new-hwaddr 01:23:45:67:ab:cd
-        Set a new hardware station address.  Read the documentation first!
-   -w  --write-EEPROM	 Actually write the new settings into the EEPROM.
- To read and write the boot BIOS extension Flash ROM
-   -B       Show the first few bytes of the ROM
-   -L FILE  Load the Flash from FILE.
-   -S FILE  Store the Flash image to FILE.
-
-   -D  --debug
-   -v  --verbose	Report each action taken.
-   -V  --version	Emit version information.
-
-   -A  --advertise <speed|setting>  (See the mii-diag manual page.)
-";
+"Usage: %s [-aDfrRvVw] [-AF <speed+duplex>] [-#<BoardNum>]\n"
+"\n"
+"   Show the internal state of a network adapter.\n"
+"\n"
+"   The common usage is\n"
+"      vortex-diag -aem\n"
+"\n"
+" Frequently used options are\n"
+"   -a  --show_all_registers	Print all registers.\n"
+"   -e  --show-eeprom	Dump EEPROM contents, \"-ee\" shows the details.\n"
+"   -m  --show_mii		Print the MII transceiver state\n"
+"					Using -mm monitors the link.\n"
+"   -f  --force		Perform operation, even on a running NIC.\n"
+"\n"
+" To operate on a single NIC, or one that hasn't been automatically found:\n"
+"   -#  --card_num	INDEX	Operate on the specified card index.\n"
+"   -p  --port-base	IOADDR	Assume an adapter at the specified I/O address.\n"
+"   -t  --chip-type	TYPE	Specify adapter type (with '-p'), use '-1' to list.\n"
+"\n"
+" To change the persistent EEPROM settings\n"
+"   -G  --parameters PARMS	Set adapter-specific parameters.\n"
+"   -H  --new-hwaddr 01:23:45:67:89:ab\n"
+"        Set a new hardware station address.  Read the documentation first!\n"
+"   -w  --write-EEPROM	 Actually write the new settings into the EEPROM.\n"
+" To read and write the boot BIOS extension Flash ROM\n"
+"   -B       Show the first few bytes of the ROM\n"
+"   -L FILE  Load the Flash from FILE.\n"
+"   -S FILE  Store the Flash image to FILE.\n"
+"\n"
+"   -D  --debug\n"
+"   -v  --verbose	Report each action taken.\n"
+"   -V  --version	Emit version information.\n"
+"\n"
+"   -A  --advertise <speed|setting>  (See the mii-diag manual page.)\n"
+"\n";
 
 #if ! defined(__OPTIMIZE__)
 #warning  You must compile this program with the correct options!
@@ -127,8 +127,8 @@ struct option longopts[] = {
 	{"show-mii",  0, 0, 'm'},	/* Dump MII management registers. */
 	{"port-base",  1, 0, 'p'},	/* Use the specified I/O address. */
 	{"quiet",	0, 0, 'q'},		/* Decrease verbosity */
-	{"reset",	0, 0, 'R'},		/* Reset the transceiver. */
 	{"restart",	0, 0, 'r'},		/* Restart autonegotiation. */
+	{"reset",	0, 0, 'R'},		/* Reset the transceiver. */
 	{"chip-type",  1, 0, 't'},	/* Assume the specified chip type index. */
 	{"test",	0, 0, 'T'},		/* Do register and SRAM test. */
 	{"verbose",	0, 0, 'v'},		/* Verbose mode */
@@ -189,7 +189,9 @@ struct pcidev_entry {
 	 0x10B7, 0x9058, 0xffff, HAS_FLASH_BUG | IS_CYCLONE, 256, vortex_diag},
 	{"3c905B Cyclone 100baseFx", "3Com 3C905B-FX",
 	 0x10B7, 0x905A, 0xffff, HAS_FLASH_BUG | IS_CYCLONE, 256, vortex_diag},
-	{"3c905C Tornado 100baseTx", 0,
+	{"3c920 (Type 920-2) Cyclone", 0,
+	 0x10B7, 0x9202, 0xffff, HAS_FLASH_BUG | IS_CYCLONE, 256, vortex_diag},
+	{"3c920 Series NIC", 0,
 	 0x10B7, 0x9200, 0xff00, HAS_FLASH_BUG | IS_CYCLONE, 256, vortex_diag},
 	{"3c980 Cyclone, server edition", 0,
 	 0x10B7, 0x9800, 0xffff, HAS_FLASH_BUG | IS_CYCLONE, 256, vortex_diag},
@@ -1054,6 +1056,11 @@ static void parse_eeprom(unsigned short *eeprom)
 	printf("   Options: %s duplex, link beat %s.\n",
 		   ee[13] & 0x8000 ? "force full" : "negotiated",
 		   ee[13] & 0x4000 ? "check disabled" : "required");
+
+	printf("   PCI bus requested settings --  minimum grant %d, "
+		   "maximum latency %d (250ns units).\n",
+		   ((ee[8] & 0x03f0)>>6)*2, (ee[8] & 0xFC00) >> 10);
+
 	if (ee[0x18]) {
 		int MediaOptions = ee[0x19];
 		printf(" PCI Subsystem IDs: Vendor %4.4x Device %4.4x.\n",
